@@ -1,7 +1,17 @@
+import warnings
 from dataclasses import dataclass
 from inspect import getfullargspec
+from itertools import zip_longest
 from typing import Optional, Type, ClassVar, Callable, Sequence, get_type_hints, List, Any, Union
+
+import typing
+
+from ..auto_cast_handler import get_auto_cast_handler_info, is_auto_cast_handler, has_auto_cast_default
 from ..exceptions import InvalidArgumentsError
+from ..translations import translate
+
+if typing.TYPE_CHECKING:
+    from ..command import Command
 
 __all__ = [
     'get_callable_arg_types',
@@ -9,7 +19,6 @@ __all__ = [
     'convert_args_to_function_parameter_types',
     'AutoCastError',
     'Param',
-    'AutoCastHandler',
     'cast_value_to_type',
 ]
 
@@ -82,7 +91,16 @@ def _get_cast_func(type_: Type) -> Callable[[Any], Any]:
 
 def _cast_arg_to_parameter_type(arg, param: Param):
     try:
-        return _get_cast_func(param.annotation)(arg)
+        info = get_auto_cast_handler_info(param.annotation)
+        if info is not None:
+            return info.handler(arg)
+        if str(param.annotation).startswith('typing.Optional['):
+            warnings.warn('[_cast_arg_to_parameter_type] cannot cast args using GenericTypes, such as Optional and Union,'
+                          ' instead create custom/use an AutoCastHandler and annotate it with that class.'
+                          ' Make sure NOT TO USE `= None` as a default to avoid this error')
+            raise RuntimeError('cannot cast Union type annotations such as Optional and Union')
+        return param.annotation(arg)
+
     except Exception as e:
         reason = None
         if isinstance(e, AutoCastError):
@@ -100,33 +118,61 @@ def cast_value_to_type(arg, type_: Type, reason: Optional[Union[str, Callable[[E
         return AutoCastResult(value=arg, param=None, reason=reason, exception=exception, casted_value=None)
 
 
-class AutoCastHandler:
-    @classmethod
-    def _handle_auto_cast(cls, value: str):
-        pass
-
-
-def convert_args_to_function_parameter_types(function: Callable, args: Sequence[str]):
+def convert_args_to_function_parameter_types(function: Callable, args: Sequence[str], origin_cmd: 'Command' = None):
     from ..message import Message
-    types = get_callable_arg_types(function, skip_self=True)
+    params = get_callable_arg_types(function, skip_self=True)
 
     # ensure parameters are typed as Message are ignored, message is automatically passed to the command before any args are
-    for i in range(len(types) - 1, -1, -1):
-        if types[i].annotation == Message:
-            del types[i]
+    for i in range(len(params) - 1, -1, -1):
+        if params[i].annotation is Message:
+            del params[i]
 
     out_args = []
     i = 0
 
-    for arg, param in zip(args, types):
-        if param.type == Param.POSITIONAL:
+    required_count = 0
+    for required_idx, param in enumerate(params):
+        if not param.has_default_value and not has_auto_cast_default(param.annotation) and param.type == Param.POSITIONAL:
+            required_count = required_idx + 1
+
+    for arg, param in zip_longest(args, params, fillvalue=None):
+        # no more params left to fulfill
+        if param is None and arg is not None:
+            break
+
+        # params are left, but no more args are remaining
+        elif arg is None and param is not None and param.type == Param.POSITIONAL:
+            if is_auto_cast_handler(param.annotation):
+                info = get_auto_cast_handler_info(param.annotation)
+
+                if not info.has_default:
+                    raise InvalidArgumentsError(
+                        reason=translate('args_does_not_fulfill_required_position_args', required_count=required_count, args_len=len(args)),
+                        cmd=origin_cmd
+                    )
+
+                out_args.append(info.default)
+                i += 1
+                continue
+
+            if not param.has_default_value:
+                raise InvalidArgumentsError(
+                    reason=translate('args_does_not_fulfill_required_position_args', required_count=required_count, args_len=len(args)),
+                    cmd=origin_cmd
+                )
+
+            out_args.append(param.default)
+            i += 1
+            continue
+
+        elif arg is not None and param is not None and param.type == Param.POSITIONAL:
             if param.annotation is not None:
                 out_args.append(_cast_arg_to_parameter_type(arg, param))
             else:
                 out_args.append(arg)
             i += 1
 
-    vararg_type = next((p for p in types if p.type == Param.VARARGS), None)
+    vararg_type = next((p for p in params if p.type == Param.VARARGS), None)
     if vararg_type is not None:
         if vararg_type.annotation is not None:
             out_args.extend(map(lambda x: _cast_arg_to_parameter_type(x, vararg_type), args[i:]))
